#!/usr/bin/env python3
"""utils

"""
import json
from json.decoder import JSONDecodeError


Type = 'type'  # param type: str, int, float, dict, list, bool
Required = 'required'  # if param required, set True
Default = 'default'  # set default value if param is None
Min = 'min'  # check min if param type is int or float
Max = 'max'  # check max if param type is int or float
Split = 'split'  # auto split str to list
In = 'in'  # check value in
Not = 'not'  # check value not


def parse_to_int(s):
    """parse unknown data to int, return None when exception

    :param s:
    :return:
    """
    try:
        return int(s)
    except ValueError or TypeError:
        return None


def parse_to_float(s):
    """parse unknown data to float, return None when exception

    :param s:
    :return:
    """
    try:
        return float(s)
    except ValueError or TypeError:
        return None


def parse_to_str(s):
    """parse object to str

    :param s:
    :return:
    """
    if isinstance(s, (dict, list)):
        return json.dumps(s, ensure_ascii=False)
    else:
        return str(s)


def json_loads(s):
    """load json from string, return None when exception

    :param s:
    :return:
    """
    if isinstance(s, (dict, list)):
        return s
    try:
        return json.loads(s)
    except JSONDecodeError or TypeError:
        return None


def parser(data: dict, pattern: dict, remove_redundant: bool = False):
    """get and parse request data

    :param data:
    :param pattern: use to define attrs of each params,
        eg: {
                "name": {"type": str, "required": true},
                "nickname": {"type": str, "default": "zhangsan"},
                "age": {"type": int, "required": true, "min": 1, "max": 200}
            }
    :param remove_redundant: remove keys in data but not in pattern
    :return:
    """
    for k, v in pattern.items():
        t = v.get(Type)
        if t is None:
            continue
        # parse request data
        if data.get(k):
            if type(data[k]) == t:
                continue
            if t == int:
                data[k] = parse_to_int(data[k])
            elif t == float:
                data[k] = parse_to_float(data[k])
            elif t == dict:
                data[k] = json_loads(data[k])
            elif t == str:
                tmp = parse_to_str(data[k])
                if v.get(Split):
                    tmp = tmp.split(v[Split])
                data[k] = tmp
            elif t == list:
                if v.get(Split) and isinstance(data[k], str):
                    data[k] = data[k].split(v[Split])
                else:
                    data[k] = json_loads(data[k])
            elif t == bool:
                if data[k] in ['true', 'True', 'TRUE', 1, '1']:
                    data[k] = True
                else:
                    data[k] = False
            if data[k] is None:
                return None, 'parse[%s] error' % k

        # check required param
        if v.get(Required) and data.get(k) is None:
            return None, '%s is required' % k

        # check num min and max
        if t == int or t == float:
            if v.get(Min):
                if not isinstance(data.get(k), (int, float)) or data[k] < v['min']:
                    return None, '%s check min invalid' % k
            if v.get(Max):
                if not isinstance(data.get(k), (int, float)) or data[k] > v['max']:
                    return None, '%s check max invalid' % k
        # check in
        if v.get(In):
            if t == list:
                if set(data[k]).difference(v[In]):
                    return None, 'key[%s] In check error' % k
            else:
                if data[k] not in data[In]:
                    return None, 'key[%s] In check error' % k
        # check not
        if v.get(Not):
            if t == list:
                if set(data[k]) & v[Not]:
                    return None, 'key[%s] Not check error' % k
            else:
                if data[k] in data[In]:
                    return None, 'key[%s] Not check error' % k

        # set default
        if data.get(k) is None and v.get(Default) is not None:
            data[k] = v[Default]

    # remove redundant in data but not in pattern
    if remove_redundant:
        redundant_keys = set(data.keys()).difference(set(pattern.keys()))
        for rk in redundant_keys:
            del data[rk]

    return data, None



#!/usr/bin/env python3
"""gunicorn config

"""
import os

from conf import app_conf


worker_class = app_conf.worker_class
if 'gevent' in worker_class:
    from gevent import monkey
    monkey.patch_all()

preload_app = True
daemon = True
loglevel = app_conf.loglevel
debug = False if loglevel != 'debug' else True
bind = '{}:{}'.format(app_conf.host, app_conf.port)
workers = app_conf.workers
threads = app_conf.threads

x_forwarded_for_header = 'X-FORWARDED-FOR'
path_of_current_file = os.path.abspath(__file__)
path_of_current_dir = os.path.split(path_of_current_file)[0]
pidfile = '%s/logs/gunicorn.pid' % path_of_current_dir
errorlog = '%s/logs/info.log' % path_of_current_dir
accesslog = '%s/logs/access.log' % path_of_current_dir



# -*- coding: utf-8 -*-
import ast
import os
import re

from setuptools import find_packages, setup


# parse version from easy_flask/__init__.py
_version_re = re.compile(r'__version__\s+=\s+(.*)')
_init_file = os.path.join(os.path.abspath(os.path.dirname(__file__)), "easy_flask", "__init__.py")
with open(_init_file, 'rb') as f:
    version = str(ast.literal_eval(_version_re.search(
        f.read().decode('utf-8')).group(1)))
filepath = 'README.md'

setup(
    name='easy-flask-restful',
    version=version,
    description="Easy to Make Flask Server",
    long_description=open(filepath, encoding='utf-8').read(),
    long_description_content_type='text/markdown',
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3",
        "Intended Audience :: Developers",
        "Intended Audience :: System Administrators",
    ],
    keywords='',
    author='Guo Tengda',
    author_email='ttguotengda@foxmail.com',
    url='https://github.com/GuoTengda1993/easy-flask',
    license='MIT',
    packages=find_packages(),
    include_package_data=True,
    package_data={'easy_flask': ['conf/config.ini', 'control.sh']},
    zip_safe=False,
    install_requires=["Flask>=1.0.0", "flask-restful>=0.3.0"],
    entry_points={
        'console_scripts': [
            'easy-flask = easy_flask:main',
        ]
    },
    data_files=[filepath]
)



#!/usr/bin/env python3
"""internal class and functions

"""
import json
from typing import Mapping, Union

from flask import g, request, Response
from flask.views import MethodView
from werkzeug.wrappers import Response as ResponseBase

from internal.error import BaseError
from utils.parser import parser


def unpack(value):
    """Return a two tuple of data and code"""
    if isinstance(value, BaseError):
        data = {
            'errno': value.errno,
            'msg': value.msg,
            'log_id': g.log_id
        }
        return data, value.code

    res = value
    code = 200
    msg = 'success'
    if isinstance(value, tuple):
        if len(value) == 2:
            res, code = value
        else:
            res, code, msg = value[0], value[1], value[2]

    data = {
        'errno': 0,
        'data': res,
        'msg': msg,
        'log_id': g.log_id
    }
    return data, code


class Resource(MethodView):
    """
    Represents an abstract RESTful resource. Concrete resources should
    extend from this class and expose methods for each supported HTTP
    method. If a resource is invoked with an unsupported HTTP method,
    the API will return a response with status 405 Method Not Allowed.
    Otherwise the appropriate method is called and passed all arguments
    from the url rule used when adding the resource to an Api instance. See
    :meth:`~flask_restful.Api.add_resource` for details.
    """
    representations = None
    method_decorators = []

    def dispatch_request(self, *args, **kwargs):
        # Token from flask-restful
        # change point: set log_id for each json response
        meth = getattr(self, request.method.lower(), None)
        if meth is None and request.method == 'HEAD':
            meth = getattr(self, 'get', None)
        assert meth is not None, 'Unimplemented method %r' % request.method

        if isinstance(self.method_decorators, Mapping):
            decorators = self.method_decorators.get(request.method.lower(), [])
        else:
            decorators = self.method_decorators

        for decorator in decorators:
            meth = decorator(meth)

        resp = meth(*args, **kwargs)

        if isinstance(resp, ResponseBase):  # There may be a better way to test
            return resp

        data, code = unpack(resp)
        res = json.dumps(data, ensure_ascii=False)
        return Response(res, status=code, content_type='application/json')

    def parse_request_data(self, pattern: Union[dict, None] = None, remove_redundant: bool = False):
        """get and parse request data

        :param pattern: use to define attrs of each params
        :param remove_redundant: remove keys in data but not in pattern
        :return:
        """
        data = dict()
        args = request.args.to_dict()
        if args:
            data.update(args)
        if request.content_type:
            if 'form' in request.content_type:
                form = request.form.to_dict()
                if form:
                    data.update(form)
            elif 'json' in request.content_type:
                body = request.json
                if body:
                    data.update(body)

        if pattern:
            data, err = parser(data=data, pattern=pattern, remove_redundant=remove_redundant)
            if err:
                return None, error.ParamsError(err)

        return data, None
